* Function.: SUMF()
* Authors..: Kent Boyer & Scott Davidson
* Date.....: November 30, 1990
* Notes....: SUMS a numeric field within a .DBF file.  The records included
* .........: can be selected by using a condition or an index key.
* .........: Parameters Passed -
* .........:      SumFld    : field to be summed                     (required)
* .........:      ModeFlag  : Entire file or Range (using index)     (optional)
* .........:      Criterion : Logical expression used to select rows (optional)
* .........:      Table     : ALIAS of .DBF file to operate on       (optional)
* .........:      IndexKey  : ALIAS of .DBF file to operate on       (optional)
* .........: Returns -
* .........:      Sum of numeric field or error flag (0.0000000001)
* .........:
* .........: Example: SUMF("InvAmt",'R',"CustID='ACME'",'Invoices','InvNo')
* .........: NOTE: The Field argument specified in quotations, as the function
* .........: needs to be passed the name of the field, not it's present value.

* NOTE:  To use WHILE feature, criterion must have the key value
*        present following the left-most occurrence of '='

FUNCTION SumF
PARAMETERS SumFld,ModeFlag,Criterion,Table,IndexKey

PRIVATE Pcnt,SumVal,AbortFlag,RangeKey,OldArea,OldOrder,OldRecno,OldRec2
Pcnt = Pcount()
AbortFlag = .F.
OldRecno = RECNO()

** Error Trapping, Parameter Handling
IF Pcnt < 1
   AbortFlag = .T.
ELSE
   IF Pcnt = 1
      ModeFlag = 'E'
      Criterion = '.T.'
      Table = ''
      IndexKey = ''
   ELSE
      ModeFlag = upper(ModeFlag)
      IF LEN(ModeFlag) > 1 .or. !ModeFlag $ 'ER'
         AbortFlag = .T.
      ELSE
         IF Pcnt = 2
            IF ModeFlag = 'R'
               AbortFlag = .T.
            ELSE
               Criterion = '.T.'
               Table = ''
               IndexKey = ''
            ENDIF
         ELSE
            IF TYPE(Criterion) # 'L'
               AbortFlag = .T.
            ELSE
               IF ModeFlag = 'R'
                  RangeKey = SUBS(Criterion,AT('=',Criterion)+1)
               ENDIF
               IF Pcnt = 3
                  Table = ''
                  IndexKey = ''
               ELSE
                  IF SELECT(Table) = 0 .AND..NOT. AbortFlag
                     AbortFlag = .T.
                  ELSE
                     OldArea = SELECT()
                     SELECT SELECT(Table)
                     OldRec2 = RECNO()
                  ENDIF
                  IF Pcnt = 4
                     IndexKey = ''
                  ELSE
                     OldOrder = INDEXORD()
                     IndexKey = UPPER(IndexKey)
                     FOR n = 1 TO 9
                        IF upper(INDEXKEY(n)) = IndexKey
                           EXIT
                        ENDIF
                     NEXT n
                     IF n = 10
                        AbortFlag = .T.
                     ELSE
                        SET ORDER TO (n)
                     ENDIF
                  ENDIF
               ENDIF
            ENDIF
         ENDIF
      ENDIF
   ENDIF
ENDIF

** Compute SUM
IF !AbortFlag
   IF TYPE(SumFld) # 'N'
      AbortFlag = .T.
   ELSE
      IF ModeFlag = 'E'
         SUM &SumFld FOR &Criterion TO SumVal
      ELSE
         SEEK &RangeKey
         SUM &SumFld WHILE &Criterion TO SumVal
      ENDIF
   ENDIF
ENDIF !AbortFlag

IF .NOT. EMPTY(IndexKey)
   SET ORDER TO (OldOrder)
ENDIF

IF .NOT. EMPTY(Table)
   SELECT (OldArea)
   GOTO OldRec2
ENDIF

IF RECNO() # OldRecno .AND. OldRecno > 0
   GO OldRecno
ENDIF

** Return Result
IF AbortFlag
   SumVal = 0.0000000001            && Return a numeric error condition
ENDIF

RETURN (SumVal)
* EOF SumF
